@@ -7,7 +7,7 @@
 #' @param y_imp A Vector with the variable to impute.
 #' @param X_imp A data.frame with the fixed effects variables.
 #' @param Z_imp A data.frame with the random effects variables.
-#' @param heap A numeric value saying to which values the data might be heaped.
+#' @param spike A numeric value saying to which values Y might be spiked
 #' @param clID A vector with the cluster ID.
 #' @param nitt An integer defining number of MCMC iterations (see MCMCglmm).
 #' @param burnin burnin A numeric value between 0 and 1 for the desired percentage of
@@ -17,7 +17,7 @@
 #' chains, that are only examined by few iterations (say less than 1000).
 #' @param pvalue A numeric between 0 and 1 denoting the threshold of p-values a variable in the imputation
 #' model should not exceed. If they do, they are excluded from the imputation model.
-#' @param rounding_degrees A numeric vector with the presumed rounding degrees.
+#' @param k An integer defining the allowed maximum of levels in a factor covariate.
 #' @return A list with 1. 'y_ret' the n x 1 data.frame with the original and imputed values.
 #' 2. 'Sol' the Gibbs-samples for the fixed effects parameters.
 #' 3. 'VCV' the Gibbs-samples for variance parameters.
@@ -26,35 +26,37 @@ imp_semicont_multi <- function(y_imp,
                              X_imp,
                              Z_imp,
                              clID,
-                             heap = NULL,
+                             spike = NULL,
                              nitt = 22000,
                              burnin = 2000,
                              thin = 20,
                              pvalue = 0.2,
-                             rounding_degrees = c(1, 10, 100, 1000)){
-  if(is.null(heap)){
-    heap <-  list_of_spikes_maker(data.frame(y_imp))$y_imp
+                             k = Inf){
+  #If no spike was given, the list_of_spikes_maker is use, which basically returns the mode.
+  if(is.null(spike)){
+    spike <-  list_of_spikes_maker(data.frame(y_imp))$y_imp
   }
 
-  if(is.null(heap)){
-    heap <- 0
+  #if for some near-to-impossible reasons, spike is still NULL, it is set to 0.
+  if(is.null(spike)){
+    spike <- 0
   }
 
   tmp_data <- cbind(y_imp, X_imp, Z_imp, clID)
   n <- nrow(tmp_data)
 
-  #these steps are neccesary, because in wrapper there is a value given for heap and max.se
+  #these steps are neccesary, because in wrapper there is a value given for spike and max.se
   #but those values could be NULL
 
   y_binary <- factor(rep(NA, length(y_imp)), levels = c(0, 1))
 
-  #The observations that are equal to the heaping value and are not NA...
-  condition0 <- (y_imp == heap) & !is.na(y_imp)
+  #The observations that are equal to the spike and are not NA...
+  condition0 <- (y_imp == spike) & !is.na(y_imp)
   #...are set to 0.
   y_binary[condition0] <- 0
 
-  #The observations that are unequal to the heaping value and are not NA...
-  condition1 <- (y_imp != heap) & !is.na(y_imp)
+  #The observations that are unequal to the spike and are not NA...
+  condition1 <- (y_imp != spike) & !is.na(y_imp)
   #...are set to 1.
   y_binary[condition1] <- 1
 
@@ -66,10 +68,10 @@ imp_semicont_multi <- function(y_imp,
                            Z_imp = Z_imp,
                            clID = clID,
                            nitt = nitt,
-                           thin = thin,
                            burnin = burnin,
+                           thin = thin,
                            pvalue = pvalue,
-                           rounding_degrees = rounding_degrees)
+                           k = k)
 
   what_method <- tmp1$y_ret
   # use the imputation function of the continuous variable to generate y1.imp
@@ -79,10 +81,10 @@ imp_semicont_multi <- function(y_imp,
                           Z_imp = Z_imp[what_method == 1, ,drop = FALSE],
                           clID = clID[what_method == 1],
                           nitt = nitt,
-                          thin = thin,
                           burnin = burnin,
+                          thin = thin,
                           pvalue = pvalue,
-                          rounding_degrees = rounding_degrees)
+                          k = k)
 
   y1_imp <- tmp2$y_ret
 
@@ -92,9 +94,9 @@ imp_semicont_multi <- function(y_imp,
   y_tmp <- array(NA, dim = length(y_imp))
   y_tmp[what_method == 1] <- y1_imp[, 1]
 
-  # the observations with method 0 (heaped observation)
-  # get the heap
-  y_tmp[what_method == 0] <- heap
+  # the observations with method 0 (spiked observation)
+  # get the spike
+  y_tmp[what_method == 0] <- spike
 
   y_ret <- data.frame(y_ret = y_tmp)
 
